@@ -89,8 +89,7 @@ extern NihList *events;
 
 void   event_init    (void);
 
-Event *event_new     (const void *parent, const char *name, char **env)
-	__attribute__ ((malloc));
+Event *event_new     (const void *parent, const char *name, char **env);
 
 void   event_block   (Event *event);
 void   event_unblock (Event *event);
@@ -98,13 +97,13 @@ void   event_unblock (Event *event);
 void   event_poll    (void);
 
 json_object *event_serialise (const Event *event)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 Event *event_deserialise (json_object *json)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 json_object  * event_serialise_all (void)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 int            event_deserialise_all (json_object *json)
 	__attribute__ ((warn_unused_result));
