@@ -31,7 +31,6 @@
 #include "session.h"
 #include "conf.h"
 #include "paths.h"
-#include <sys/prctl.h>
 
 static int  log_file_open   (Log *log);
 static int  log_file_write  (Log *log, const char *buf, size_t len);
@@ -687,6 +686,9 @@ log_read_watch (Log *log)
 		 * causes the loop to be exited.
 		 */
 		if (len <= 0) {
+			if (saved == EINTR)
+				continue;
+
 			/* Job process has ended and we've drained all the data the job
 			 * produced, so remote end must have closed.
 			 *
@@ -794,6 +796,11 @@ log_clear_unflushed (void)
 		elem = (NihListEntry *)iter;
 		log = elem->data;
 
+		/* To be added to this list, log should have been
+		 * detached from its parent job.
+		 */
+		nih_assert (log->detached);
+
 		/* We expect 'an' error (as otherwise why would the log be
 		 * in this list?), but don't assert EROFS specifically
 		 * as a precaution (since an attempt to flush the log at
@@ -801,9 +808,19 @@ log_clear_unflushed (void)
 		 */
 		nih_assert (log->open_errno);
 
-		nih_assert (log->unflushed->len);
-		nih_assert (log->remote_closed);
-		nih_assert (log->detached);
+		if (log->remote_closed) {
+			/* Parent job has ended and unflushed data
+			 * exists.
+			 */
+			nih_assert (log->unflushed->len);
+		} else {
+			/* Parent job itself has ended, but job spawned one or
+			 * more processes that are still running and
+			 * which might still produce output (the error
+			 * handler has therefore not been called).
+			 */
+			nih_assert (log->io);
+		}
 
 		if (log_file_open (log) != 0)
 			return -1;
@@ -811,6 +828,7 @@ log_clear_unflushed (void)
 		if (log_file_write (log, NULL, 0) < 0)
 			return -1;
 
+		/* This will handle any remaining unflushed log data */
 		nih_free (log);
 	}
 
@@ -838,14 +856,8 @@ log_serialise (Log *log)
 	if (! json)
 		return NULL;
 
-	if (! log || (! log->io && log->unflushed && ! log->unflushed->len)) {
-		/* Create a "placeholder" log object for non-existent
-		 * log objects and for those that are no longer usable.
-		 */
-		if (! state_set_json_string_var (json, "path", NULL))
-			goto error;
-		return json;
-	}
+	if (! log || (! log->io && log->unflushed && ! log->unflushed->len))
+		goto placeholder;
 
 	/* Attempt to flush any cached data */
 	if (log->unflushed && log->unflushed->len) {
@@ -858,7 +870,12 @@ log_serialise (Log *log)
 			(void)log_file_write (log, NULL, 0);
 	}
 
-	nih_assert (log->io);
+	/* Job associated with log has ended. If we failed to write
+	 * unflushed data above, it will now be lost as we cannot
+	 * create a valid serialisation without an associated NihIo.
+	 */
+	if (! log->io)
+		goto placeholder;
 
 	if (! state_set_json_int_var_from_obj (json, log, fd))
 		goto error;
@@ -902,6 +919,14 @@ log_serialise (Log *log)
 
 	return json;
 
+placeholder:
+	/* Create a "placeholder" log object for non-existent
+	 * log objects and for those that are no longer usable.
+	 */
+	if (! state_set_json_string_var (json, "path", NULL))
+		goto error;
+	return json;
+
 error:
 	json_object_put (json);
 	return NULL;
