@@ -240,7 +240,12 @@ static int stanza_usage       (JobClass *class, NihConfigStanza *stanza,
 			       size_t *pos, size_t *lineno)
 	__attribute__ ((warn_unused_result));
 
-
+/**
+ * debug_stanza_enabled:
+ *
+ * If TRUE, honour the debug stanza, else consider it a NOP.
+ **/
+int debug_stanza_enabled = FALSE;
 
 /**
  * stanzas:
@@ -1084,7 +1089,8 @@ stanza_debug (JobClass           *class,
 	nih_assert (file != NULL);
 	nih_assert (pos != NULL);
 
-	class->debug = TRUE;
+	if (debug_stanza_enabled)
+		class->debug = TRUE;
 
 	return 0;
 }
