@@ -51,10 +51,10 @@ extern json_object *json_sessions;
 
 /* Prototypes for static functions */
 static json_object  *session_serialise   (const Session *session)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 static Session *session_deserialise (json_object *json)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 
 /**
@@ -68,7 +68,7 @@ NihList *sessions = NULL;
 /**
  * disable_sessions:
  *
- * If TRUE, disable user and chroot sessions, resulting in a
+ * If TRUE, disable chroot sessions, resulting in a
  * "traditional" (pre-session support) system.
  **/
 int disable_sessions = FALSE;
@@ -89,25 +89,35 @@ session_init (void)
 		sessions = NIH_MUST (nih_list_new (NULL));
 }
 
+/**
+ * session_destroy:
+ *
+ * Clean up the sessions list.
+ **/
+void
+session_destroy (void)
+{
+	if (sessions)
+		nih_free (sessions);
+}
+
 
 /**
  * session_new:
  * @parent: parent,
- * @chroot: full chroot path,
- * @user: user id.
+ * @chroot: full chroot path.
  *
- * Create a new session.
+ * Create a new chroot session.
  *
  * Returns: new Session, or NULL on error.
  **/
 Session *
 session_new (const void *parent,
-	     const char *chroot,
-	     uid_t       user)
+	     const char *chroot)
 {
 	Session *session;
 
-	nih_assert ((chroot != NULL) || (user != 0));
+	nih_assert (chroot);
 
 	session_init ();
 
@@ -117,18 +127,12 @@ session_new (const void *parent,
 
 	nih_list_init (&session->entry);
 
-	if (chroot) {
-		session->chroot = nih_strdup (session, chroot);
-		if (! session->chroot) {
-			nih_free (session);
-			return NULL;
-		}
-	} else {
-		session->chroot = NULL;
+	session->chroot = nih_strdup (session, chroot);
+	if (! session->chroot) {
+		nih_free (session);
+		return NULL;
 	}
 
-	session->user = user;
-
 	session->conf_path = NULL;
 
 	nih_alloc_set_destructor (session, nih_list_destroy);
@@ -151,14 +155,13 @@ Session *
 session_from_dbus (const void     *parent,
 		   NihDBusMessage *message)
 {
-	const char     *sender;
-	DBusError       dbus_error;
-	unsigned long   unix_user;
-	unsigned long   unix_process_id;
-	char            root[PATH_MAX];
-	Session        *session;
-	struct passwd  *pwd;
-	nih_local char *conf_path = NULL;
+	DBusError        dbus_error;
+	unsigned long    unix_process_id;
+	char             root[PATH_MAX];
+	Session         *session;
+	nih_local char  *conf_path = NULL;
+	nih_local char  *symlink = NULL;
+	ssize_t          len;
 
 	nih_assert (message != NULL);
 
@@ -170,126 +173,43 @@ session_from_dbus (const void     *parent,
 
 	session_init ();
 
-	/* Ask D-Bus nicely for the origin uid and/or pid of the caller;
-	 * sadly we can't ask the bus daemon for the origin pid, so that
-	 * one will just have to stay user-session only.
-	 */
 	dbus_error_init (&dbus_error);
 
-	sender = dbus_message_get_sender (message->message);
-	if (sender) {
-		unix_user = dbus_bus_get_unix_user (message->connection, sender,
-						    &dbus_error);
-		if (unix_user == (unsigned long)-1) {
-			dbus_error_free (&dbus_error);
-			unix_user = 0;
-		}
-
-		unix_process_id = 0;
-
-	} else {
-		if (! dbus_connection_get_unix_user (message->connection,
-						     &unix_user))
-			unix_process_id = 0;
-
-		if (! dbus_connection_get_unix_process_id (message->connection,
-							   &unix_process_id))
-			unix_process_id = 0;
-	}
-
-	/* If we retrieved a process id, look up the root path for it;
-	 * if it's just '/' don't worry so much about it.
-	 */
-	if (unix_process_id) {
-		nih_local char *symlink = NULL;
-		ssize_t len;
-
-		symlink = NIH_MUST (nih_sprintf (NULL, "/proc/%lu/root",
-						 unix_process_id));
-		len = readlink (symlink, root, sizeof root);
-		if (len < 0)
-			return NULL;
-
-		root[len] = '\0';
-
-		if (! strcmp (root, "/")) {
-			unix_process_id = 0;
-			if (! unix_user)
-				return NULL;
-		}
-
-	} else if (! unix_user) {
-		/* No process id or user id found, return the NULL session */
+	/* Query origin pid of the caller */
+	if (! dbus_connection_get_unix_process_id (message->connection,
+				&unix_process_id)) {
 		return NULL;
 	}
 
-	if (unix_user) {
-		pwd = getpwuid (unix_user);
+	/* Look up the root path for retrieved pid */
+	symlink = NIH_MUST (nih_sprintf (NULL, "/proc/%lu/root",
+				unix_process_id));
+	len = readlink (symlink, root, sizeof (root)-1);
+	if (len < 0)
+		return NULL;
 
-		if (! pwd || ! pwd->pw_dir) {
-			nih_error ("%lu: %s: %s", unix_user,
-				   _("Unable to lookup home directory"),
-				   strerror (errno));
-			return NULL;
-		}
-
-		NIH_MUST (nih_strcat_sprintf (&conf_path, NULL, "%s/%s",
-					pwd->pw_dir, USERCONFDIR));
-	}
+	root[len] = '\0';
 
+	/* Path is not inside a chroot */
+	if (! strcmp (root, "/"))
+		return NULL;
 
 	/* Now find in the existing Sessions list */
 	NIH_LIST_FOREACH_SAFE (sessions, iter) {
 		Session *session = (Session *)iter;
 
-		if (unix_process_id) {
-			if (! session->chroot)
-				continue;
-
+		if (strcmp (session->chroot, root))
 			/* ignore sessions relating to other chroots */
-			if (strcmp (session->chroot, root))
-				continue;
-		}
-
-		/* ignore sessions relating to other users */
-		if (unix_user != session->user)
 			continue;
 
-		/* Found a user with the same uid but different
-		 * conf_path to the existing session user. Either the
-		 * original user has been deleted and a new user created
-		 * with the same uid, or the original users home
-		 * directory has changed since they first started
-		 * running jobs. Whatever the reason, we (can only) honour
-		 * the new value.
-		 *
-		 * Since multiple users with the same uid are considered
-		 * to be "the same user", invalidate the old path,
-		 * allowing the correct new path to be set below.
-		 *
-		 * Note that there may be a possibility of trouble if
-		 * the scenario relates to a deleted user and that original
-		 * user still has jobs running. However, if that were the
-		 * case, those jobs would likely fail anyway since they
-		 * would have no working directory due to the original
-		 * users home directory being deleted/changed/made inaccessible.
-		 */
-		if (unix_user && conf_path && session->conf_path &&
-				strcmp (conf_path, session->conf_path)) {
-			nih_free (session->conf_path);
-			session->conf_path = NULL;
-		}
-
 		if (! session->conf_path)
 			session_create_conf_source (session, FALSE);
 
 		return session;
 	}
 
-
 	/* Didn't find one, make a new one */
-	session = NIH_MUST (session_new (parent, unix_process_id ? root : NULL,
-					 unix_user));
+	session = NIH_MUST (session_new (parent, root));
 	session_create_conf_source (session, FALSE);
 
 	return session;
@@ -317,27 +237,8 @@ session_create_conf_source (Session *session, int deserialised)
 	session_init ();
 
 	if (! deserialised) {
-		if (session->chroot)
-			session->conf_path = NIH_MUST (nih_strdup (NULL, session->chroot));
-		if (session->user) {
-			struct passwd *pwd;
-
-			pwd = getpwuid (session->user);
-			if (! pwd) {
-				nih_error ("%d: %s: %s", session->user,
-						_("Unable to lookup home directory"),
-						strerror (errno));
-
-				nih_free (session->conf_path);
-				session->conf_path = NULL;
-				return;
-			}
-
-			NIH_MUST (nih_strcat_sprintf (&session->conf_path, NULL, "%s/%s",
-						pwd->pw_dir, USERCONFDIR));
-		} else {
-			NIH_MUST (nih_strcat (&session->conf_path, NULL, CONFDIR));
-		}
+		session->conf_path = NIH_MUST (nih_strdup (NULL, session->chroot));
+		NIH_MUST (nih_strcat (&session->conf_path, NULL, CONFDIR));
 	}
 
 	source = NIH_MUST (conf_source_new (session, session->conf_path,
@@ -375,9 +276,10 @@ session_serialise (const Session *session)
 	json_object  *json;
 	json_object  *conf_path = NULL;
 	json_object  *chroot = NULL;
-	json_object  *user;
 
 	nih_assert (session);
+	nih_assert (session->chroot);
+	nih_assert (session->conf_path);
 
 	session_init ();
 
@@ -385,26 +287,16 @@ session_serialise (const Session *session)
 	if (! json)
 		return NULL;
 
-	if (session->chroot) {
-		chroot = json_object_new_string (session->chroot);
-		if (! chroot)
-			goto error;
-	}
+	chroot = json_object_new_string (session->chroot);
+	if (! chroot)
+		goto error;
 
 	json_object_object_add (json, "chroot", chroot);
 
-	user = state_new_json_int (session->user);
-	if (! user)
+	conf_path = json_object_new_string (session->conf_path);
+	if (! conf_path)
 		goto error;
 
-	json_object_object_add (json, "user", user);
-
-	if (session->conf_path) {
-		conf_path = json_object_new_string (session->conf_path);
-		if (! conf_path)
-			goto error;
-	}
-
 	json_object_object_add (json, "conf_path", conf_path);
 
 	return json;
@@ -459,6 +351,9 @@ error:
  *
  * Convert @json into a Session object.
  *
+ * NOTE: Any user sessions seen when re-execing from an older version
+ * of Upstart are implicitly ignored.
+ *
  * Returns: Session object, or NULL on error.
  **/
 static Session *
@@ -466,7 +361,6 @@ session_deserialise (json_object *json)
 {
 	Session              *session = NULL;
 	nih_local const char *chroot = NULL;
-	uid_t                 user;
 
 	nih_assert (json);
 
@@ -477,11 +371,11 @@ session_deserialise (json_object *json)
 	if (! state_get_json_string_var (json, "chroot", NULL, chroot))
 		return NULL;
 
-	if (! state_get_json_int_var (json, "user", user))
+	if (! chroot)
 		return NULL;
 
 	/* Create a new session */
-	session = NIH_MUST (session_new (NULL, chroot, user));
+	session = NIH_MUST (session_new (NULL, chroot));
 
 	if (! state_get_json_string_var_to_obj (json, session, conf_path))
 		goto error;
@@ -535,7 +429,16 @@ session_deserialise_all (json_object *json)
 		if (! session)
 			goto error;
 
-		session_create_conf_source (session, TRUE);
+		/* Note that we do not call session_create_conf_source()
+		 * to create a conf source for each session.
+		 *
+		 * Once we support the deserialisation of chroots, this
+		 * will need to happen. However, currently making this
+		 * call will result in the creation of JobClasses for
+		 * each ConfFile associated with a Session. This is
+		 * pointless since those session-specific JobClasses
+		 * are never used.
+		 */
 	}
 
 	return 0;
@@ -605,7 +508,10 @@ session_from_index (int idx)
 
 		if (i == idx)
 			return session;
+		++i;
 	}
 
+	/* TODO xnox 20130719 shouldn't we return -1 on error, just
+	 * like session_get_index() above? */
 	nih_assert_not_reached ();
 }
