@@ -54,6 +54,7 @@ json_object *json_classes = NULL;
 json_object *json_conf_sources = NULL;
 
 extern char *log_dir;
+extern char *control_bus_address;
 
 /**
  * args_copy:
@@ -340,6 +341,7 @@ state_to_string (char **json_string, size_t *len)
 {
 	json_object  *json;
 	json_object  *json_job_environ;
+	json_object  *json_control_bus_address;
 	const char   *value;
 
 	nih_assert (json_string);
@@ -366,6 +368,20 @@ state_to_string (char **json_string, size_t *len)
 
 	json_object_object_add (json, "events", json_events);
 
+	json_control_bus_address = control_serialise_bus_address ();
+
+	/* Take care to distinguish between memory failure and an
+	 * as-yet-not-set control bus address.
+	 */
+	if (! json_control_bus_address && control_bus_address) {
+		nih_error ("%s %s",
+				_("Failed to serialise"),
+			       _("control bus address"));
+		goto error;
+	}
+
+	json_object_object_add (json, "control_bus_address", json_control_bus_address);
+
 	json_job_environ = job_class_serialise_job_environ ();
 
 	if (! json_job_environ) {
@@ -427,6 +443,7 @@ state_from_string (const char *state)
 	int                       ret = -1;
 	json_object              *json;
 	json_object              *json_job_environ;
+	json_object              *json_control_bus_address;
 	enum json_tokener_error   error;
 
 	nih_assert (state);
@@ -458,6 +475,20 @@ state_from_string (const char *state)
 		goto out;
 	}
 
+	ret = json_object_object_get_ex (json, "control_bus_address", &json_control_bus_address);
+
+	if (json_control_bus_address) {
+		if (control_deserialise_bus_address (json_control_bus_address) < 0) {
+			nih_error ("%s control details", _("Failed to deserialise"));
+			goto out;
+		}
+	} else if (! ret) {
+		/* Probably deserialising from older format that doesn't
+		 * encode control details.
+		 */
+		nih_warn ("%s", _("No control details present in state data"));
+	}
+
 	/* Again, we cannot error here since older JSON state data did
 	 * not encode ConfSource or ConfFile objects.
 	 */
