@@ -1569,6 +1569,16 @@ test_start (void)
 	TEST_EQ (job->goal, JOB_START);
 	TEST_EQ (job->state, JOB_STARTING);
 
+	TEST_EQ_P (job->start_env, NULL);
+	TEST_EQ_P (job->stop_env, NULL);
+
+	TEST_NE_P (job->env, NULL);
+	TEST_EQ_STRN (job->env[0], "PATH=");
+	TEST_EQ_STRN (job->env[1], "TERM=");
+	TEST_EQ_P (job->env[2], NULL);
+
+	TEST_NE_P (job->env, env);
+
 	TEST_LIST_NOT_EMPTY (&job->blocking);
 
 	blocked = (Blocked *)job->blocking.next;
@@ -1664,6 +1674,15 @@ test_start (void)
 
 	TEST_LIST_EMPTY (&job->blocking);
 
+	TEST_EQ_P (job->start_env, NULL);
+	TEST_EQ_P (job->stop_env, NULL);
+
+	TEST_NE_P (job->env, NULL);
+	TEST_EQ_STRN (job->env[0], "PATH=");
+	TEST_EQ_STRN (job->env[1], "TERM=");
+	TEST_EQ_P (job->env[2], NULL);
+
+	TEST_NE_P (job->env, env);
 
 	dbus_connection_flush (conn);
 
@@ -1746,6 +1765,16 @@ test_start (void)
 	TEST_EQ (job->goal, JOB_START);
 	TEST_EQ (job->state, JOB_STOPPING);
 
+	TEST_EQ_P (job->env, NULL);
+	TEST_EQ_P (job->stop_env, NULL);
+
+	TEST_NE_P (job->start_env, NULL);
+	TEST_EQ_STRN (job->start_env[0], "PATH=");
+	TEST_EQ_STRN (job->start_env[1], "TERM=");
+	TEST_EQ_P (job->start_env[2], NULL);
+
+	TEST_NE_P (job->start_env, env);
+
 	TEST_LIST_NOT_EMPTY (&job->blocking);
 
 	blocked = (Blocked *)job->blocking.next;
@@ -1906,12 +1935,18 @@ test_start (void)
 	TEST_EQ (job->goal, JOB_START);
 	TEST_EQ (job->state, JOB_STARTING);
 
+	TEST_NE_P (job->start_env, env);
+	TEST_NE_P (job->stop_env, env);
+
+	TEST_NE_P (job->env, NULL);
 	TEST_EQ_STRN (job->env[0], "PATH=");
 	TEST_EQ_STRN (job->env[1], "TERM=");
 	TEST_EQ_STR (job->env[2], "FOO=wibble");
 	TEST_EQ_STR (job->env[3], "BAR=wobble");
 	TEST_EQ_P (job->env[4], NULL);
 
+	TEST_NE_P (job->env, env);
+
 	TEST_LIST_NOT_EMPTY (&job->blocking);
 
 	blocked = (Blocked *)job->blocking.next;
@@ -2087,6 +2122,14 @@ test_stop (void)
 
 	TEST_NOT_FREE (job);
 
+	TEST_EQ_P (job->env, NULL);
+	TEST_EQ_P (job->start_env, NULL);
+
+	TEST_NE_P (job->stop_env, NULL);
+	TEST_EQ_P (job->stop_env[0], NULL);
+
+	TEST_NE_P (job->stop_env, env);
+
 	TEST_EQ (job->goal, JOB_STOP);
 	TEST_EQ (job->state, JOB_STOPPING);
 
@@ -2177,6 +2220,13 @@ test_stop (void)
 
 	TEST_LIST_EMPTY (&job->blocking);
 
+	TEST_EQ_P (job->env, NULL);
+	TEST_EQ_P (job->start_env, NULL);
+
+	TEST_NE_P (job->stop_env, NULL);
+	TEST_EQ_P (job->stop_env[0], NULL);
+
+	TEST_NE_P (job->stop_env, env);
 
 	dbus_connection_flush (conn);
 
@@ -2354,10 +2404,16 @@ test_stop (void)
 	TEST_EQ (job->goal, JOB_STOP);
 	TEST_EQ (job->state, JOB_STOPPING);
 
+	TEST_EQ_P (job->env, NULL);
+	TEST_EQ_P (job->start_env, NULL);
+
+	TEST_NE_P (job->stop_env, NULL);
 	TEST_EQ_STR (job->stop_env[0], "FOO=wibble");
 	TEST_EQ_STR (job->stop_env[1], "BAR=wobble");
 	TEST_EQ_P (job->stop_env[2], NULL);
 
+	TEST_NE_P (job->stop_env, env);
+
 	TEST_LIST_NOT_EMPTY (&job->blocking);
 
 	blocked = (Blocked *)job->blocking.next;
@@ -2527,6 +2583,16 @@ test_restart (void)
 	TEST_EQ (job->goal, JOB_START);
 	TEST_EQ (job->state, JOB_STOPPING);
 
+	TEST_EQ_P (job->env, NULL);
+	TEST_EQ_P (job->stop_env, NULL);
+
+	TEST_NE_P (job->start_env, NULL);
+	TEST_EQ_STRN (job->start_env[0], "PATH=");
+	TEST_EQ_STRN (job->start_env[1], "TERM=");
+	TEST_EQ_P (job->start_env[2], NULL);
+
+	TEST_NE_P (job->start_env, env);
+
 	TEST_LIST_NOT_EMPTY (&job->blocking);
 
 	blocked = (Blocked *)job->blocking.next;
@@ -2626,6 +2692,16 @@ test_restart (void)
 	TEST_EQ (job->goal, JOB_START);
 	TEST_EQ (job->state, JOB_STOPPING);
 
+	TEST_EQ_P (job->env, NULL);
+	TEST_EQ_P (job->stop_env, NULL);
+
+	TEST_NE_P (job->start_env, NULL);
+	TEST_EQ_STRN (job->start_env[0], "PATH=");
+	TEST_EQ_STRN (job->start_env[1], "TERM=");
+	TEST_EQ_P (job->start_env[2], NULL);
+
+	TEST_NE_P (job->start_env, env);
+
 	TEST_LIST_EMPTY (&job->blocking);
 
 
